#include <pic.h>
#include "lcd.h"
#include "i2c_hwd.h"
#define TH 2	//pointer THYST
#define TS 3	//pointer TSET
struct{
   char tms:1;
   char fl:1;
   char flt1:1;

}bits;
#define tms bits.tms
#define fl bits.fl
#define flt1 bits.flt1

unsigned int mSec,uSec;

void wys_hex(unsigned char dana, unsigned char poz);
void DispDec(unsigned char data,char x,char y);
unsigned int ReadTemp(void);
void WriteConfig(unsigned char config);
void WriteTemp(unsigned char pointer, unsigned int temp);
void DispTemp(unsigned int temp,char x, char y);
unsigned int KonwTemp(unsigned int temp);
void DefLcdChar(void);
//******************************************************************
//program obsugi termometru MCP9801
//interfejs I2C- sprztowy 
//kompilator Hi-tech lite
//******************************************************************

//obsuga przerwnia od Timer0 
static void interrupt
isr(void)         // przerwanie od Timer0
{
   if(T0IF){             //zerowan ie flagi przerwania
      TMR0+=131;         //8MHz/4/16/125=1kHz T=1msek
      if(tms){
      --mSec;
      if(mSec==0)
      tms=0;
      }
      T0IF = 0;         // clear the interrupt flag
   }
}
void main(void){
	OPTION=0b00000011;//ustawienie TMR0
	T0IE = 1;		// odblokowanie przerwania od przepelnienia TMR0
	GIE = 1;		// Global interrupt enable
	ANSELH=0;		//PORTB cyfrowy 
	TRISD=0;
	OpenI2C();		//incjalizacja interfejsu I2C
	InicLcd();		//inicjalizacja wyietlacza 
	DefLcdChar();
	PosLcd(1,1);
	DispLcd("Pomiar MCP9801");
	WriteConfig(0);	//zap[isanie konfiguracji 
	WriteTemp(TH,50);	//25 st C
	WriteTemp(TS,51);	//25.5 st.C
	WPUB0=1;
	while(1){
	PosLcd(11,2);
	if(RB0==0)
	DispLcd("ALERT");
	else
	DispLcd("     ");
	delay_ms(300);
	DispTemp(ReadTemp(),1,2);
	delay_ms(300);
	}
	
	}
/*************************************************************
funkcja odczytania temperatury otoczenia dla
9 bitowej rodzielczoci pomiaru
/*************************************************************/
unsigned int ReadTemp(){
unsigned int data=0;
char ack;
	i2c_start();
	ack=i2c_write(0x90);	//adres slave R/W=0
	i2c_write(0);    	//pointer TA (otoczenia)
	i2c_start();		//powtrzony start
	i2c_write(0x91);	//adres slave R/W=1
	data=i2c_read(0);	//starsze 8 bitw TA
	data=data<<8;
	data=data|i2c_read(1);//modsze 8 bitw TA z bitem NAK
	i2c_stop();
//	data>>=7;
	return(data>>7);	//bity od pozycji b0
}
/*************************************************************
funkcja zapisania rejestru konfiguracyjnego CONFIG
/*************************************************************/
void WriteConfig(unsigned char config){
char ack;
	i2c_start();
	ack=i2c_write(0x90);	//adres slave R/W=0
	ack=i2c_write(1);		//pointer CONFIG
	ack=i2c_write(config);
	i2c_stop();
}
/*************************************************************
funkcja zapisania temperatury TSET i THYST
temperatura zawarta jest w argumencie temp
/*************************************************************/
void WriteTemp(unsigned char pointer, unsigned int temp){
	temp=temp<<7;
	i2c_start();
	i2c_write(0x90);	//adres slave R/W=0
	i2c_write(pointer);		//pointer CONFIG
	i2c_write(temp>>8);
	i2c_write(temp);
	i2c_stop();
}
/*************************************************************
funkcja konwersji temperatury na warto bezwzgldn  
zwracana jest 8 bitowa warto bezwzgldna ze z informacj o czci uamkowej 
na 9-tym bicie
/*************************************************************/
unsigned int KonwTemp(unsigned int temp){
unsigned char u,t;
unsigned int Tmp=0;
	t=temp;			//w zmiennej t tylko 8 modszych bitw ( konwersja typw)
	if((temp&0x100)==0x100) //czy warto ujemna
	t=(~t)+1;		//konwersja U2 na warto bezwzgldn 
	u=t%2;			// dzielenie modulo 2	
	t=t/2;			//modu wartoci cakowitej temperatury		 
	if(u==1)
	Tmp=Tmp|0x0100; //b8=1  jest cz uamkowa 0,5C
	Tmp=Tmp|t;		//dopisanie moduu wartoci cakowitej temperatury
	return(Tmp);
}

/**************************************************
funkcja wywietlania temperatury na wywietlaczu 
alfanumerycznym. Wywietlane s 2 cyfry czci cakowitej
(zakres 0099) i cz uamkowa 0,5C
/**************************************************/
void DispTemp(unsigned int temp,unsigned char x,unsigned char y)
{
unsigned int tmp;
	PosLcd(x,y);
	WriteRd('T');WriteRd('='); 	//wywietlanie tekstu T=
	if((temp&0x0100)==0)	//sprawdzenie znaku temperatury
	WriteRd('+');
	else
	WriteRd('-');
	tmp=KonwTemp(temp);		//wykonanie konwersji 
	DispDec(tmp,x+3,y);		//wywietlenie 2 cyfr czci cakowitej
	WriteRd(',');			//wywietlenie znaku przecinka
	if((tmp&0xff00)>0)		//sprawdzenie czy jest cz uamkowa
	WriteRd(0x35);			//wywietlenie znaku 5  cze uamkowa
	else
	WriteRd(0x30);			//wywietlenie znaku 0
	WriteRd(0);			//wywietlenie znaku stopnia
	WriteRd('C');
}
//wysietlenie wartoci data w postaci 2 cyfr dziesietnie 00hex....99hex 
void DispDec(unsigned char data,char x, char y){
	PosLcd(x,y);
	WriteRd((data/10)+0x30);
    WriteRd((data%10)+0x30);
	 	
}
//zdefiniowanie znaku stopnia
void DefLcdChar(void)
{
	WriteRi(0x40);//adres 00 w pamieci CG RAM
	WriteRd(0x06);
	WriteRd(0x09);
	WriteRd(0x09);
	WriteRd(0x06);
	WriteRd(0x00);
	WriteRd(0x00);
	WriteRd(0x00);
	WriteRd(0x00);
}
void delay_ms(unsigned int msek){
	mSec=msek;
	TMR0|=131;
	tms=1;
	while(tms);
}

