
#include <pic.h>
#include "lcd.h"

#define DQ TRISB0
#define DQ_IN RB0

void delay_ms(unsigned int msek);
void delay_us(unsigned char usek);
void DispDec(unsigned char data, char x, char y );
void DispTemp(unsigned int temp,unsigned char x,unsigned char y);
void DefLcdChar(void);
unsigned int KonwTemp(unsigned int temp);
void BusReset(void);
void BusWrite(unsigned char data);
unsigned char BusRead(void);

unsigned char  tH,tL,Conf;
unsigned int mSec;
char tms;
struct{
   char thi:1;
   char tlo:1;
}bits;
#define thi bits.thi
#define tlo bits.tlo
//obsuga przerwnia od Timer0 
static void interrupt
isr(void)         // przerwanie od Timer0
{
   if(T0IF){             //zerowan ie flagi przerwania
      TMR0+=131;         //8MHz/4/16/125=1kHz T=1msek
      if(tms){
      --mSec;
      if(mSec==0)
      tms=0;
      }
      T0IF = 0;         // clear the interrupt flag
   }
}
unsigned int ReadDS(void)
{ 
	unsigned char  busy=0, temp1, temp2; 
	unsigned int temp3; 

	BusReset(); 
	BusWrite(0xCC); 
	BusWrite(0x44); 
	delay_ms(200); 
	while (busy == 0) 
	busy = BusRead(); 

	BusReset(); 
	BusWrite(0xCC); 
	BusWrite(0xBE); 
	delay_ms(10);
	temp1 = BusRead();
	temp2 = BusRead();
	tH=BusRead(); 
	tL=BusRead(); 
	Conf=BusRead(); 
	temp3=(temp2<<8)|temp1;
	
	temp3>>=3;
	delay_ms(200); 
	return(temp3); 
} 
void main(void) 
{ 
int tr; 
//unsigned char tLo, tHi;
unsigned char TH, TL;
unsigned char temp[11];
	OPTION=0b00000011;//ustawienie TMR0
	T2CON=0;//ustawienie TMR2
	ANSELH=0;
	T0IE = 1;		// odblokowanie przerwania od przepelnienia TMR0
	GIE = 1;		// odblokowanie systemu przerwa
	TRISD=0;
	DQ=1;
	DQ_IN=0;
	RC0=1;
	TRISC0=0;
	
	TL=23*2;
	TH=24*2;
	thi=tlo=1;
	InicLcd();
	DefLcdChar();


	BusWrite(0xCC);//komenda SKIP ROM 
	BusWrite(0x4E); //komenda Write Scratchpad
	BusWrite(125); //temperatura TH
	BusWrite(-55); //temperatura TL
	BusWrite(127); //rejestr konfiguracji - rodzielczoc 12bitw

	BusReset(); 
	BusWrite(0xCC);//komenda SKIP ROM 
	BusWrite(0x48);//komenda Copy Scratchpad 
	delay_ms(15); 



	while (1) 
	{ 
	tr=ReadDS();
	DispTemp(tr,1,1);
	if(tr<=TL&&thi==1)
	{tlo=1; thi=0;
	PosLcd(1,2);
	 DispLcd("TL - zalacz");
	}
	
	if(tr>=TH&&tlo==1)
	{tlo=0; thi=1;
	PosLcd(1,2);
	 DispLcd("TH - wylacz");
	}
	delay_ms(10);

	} 

} 
/*************************************************************
funkcja konwersji temperatury na warto bezwzgldn  
zwracana jest 8 bitowa warto bezwzgldna ze z informacj o czci uamkowej 
na 9-tym bicie
/*************************************************************/
unsigned int KonwTemp(unsigned int temp){
unsigned char u,t;
unsigned int Tmp=0;
	t=temp;			//w zmiennej t tylko 8 modszych bitw ( konwersja typw)
	if((temp&0x100)==0x100) //czy warto ujemna
	t=(~t)+1;		//konwersja U2 na warto bezwzgldn 
	u=t%2;			// dzielenie modulo 2	
	t=t/2;			//modu wartoci cakowitej temperatury		 
	if(u==1)
	Tmp=Tmp|0x0100; //b8=1  jest cz uamkowa 0,5C
	Tmp=Tmp|t;		//dopisanie moduu wartoci cakowitej temperatury
	return(Tmp);
}
/**************************************************
funkcja wywietlania temperatury na wywietlaczu 
alfanumerycznym. Wywietlane s 2 cyfry czci cakowitej
(zakres 0099) i cz uamkowa 0,5C
/**************************************************/
void DispTemp(unsigned int temp,unsigned char x,unsigned char y)
{
unsigned int tmp;
	PosLcd(x,y);
	WriteRd('T');WriteRd('='); 	//wywietlanie tekstu T=
	if((temp&0x0100)==0)	//sprawdzenie znaku temperatury
	WriteRd('+');
	else
	WriteRd('-');
	tmp=KonwTemp(temp);		//wykonanie konwersji 
	DispDec(tmp,x+3,y);		//wywietlenie 2 cyfr czci cakowitej
	WriteRd(',');			//wywietlenie znaku przecinka
	if((tmp&0xff00)>0)		//sprawdzenie czy jest cz uamkowa
	WriteRd(0x35);			//wywietlenie znaku 5  cze uamkowa
	else
	WriteRd(0x30);			//wywietlenie znaku 0
	WriteRd(0);			//wywietlenie znaku stopnia
	WriteRd('C');
}
//wysietlenie wartoci data w postaci 2 cyfr dziesietnie 00hex....99hex 
void DispDec(unsigned char data,char x, char y){
	PosLcd(x,y);
	WriteRd((data/10)+0x30);
    WriteRd((data%10)+0x30);
	 	
}
//zdefiniowanie znaku stopnia
void DefLcdChar(void)
{
	WriteRi(0x40);//adres 00 w pamieci CG RAM
	WriteRd(0x06);
	WriteRd(0x09);
	WriteRd(0x09);
	WriteRd(0x06);
	WriteRd(0x00);
	WriteRd(0x00);
	WriteRd(0x00);
	WriteRd(0x00);
}


void BusReset() // tylko dla jednego ukadu poaczonego do magistrali 
{ 
	DQ=0;//wystawienie stanu niskiego na magistrali 	
	delay_us( 100 );delay_us( 100 );delay_us( 100 );
	delay_us( 100 );delay_us( 100 ); // przez 500us 
	DQ=1;//DQ w stan wysokiej impedancji (wejcie) 
	delay_us( 100 );delay_us( 100 );delay_us( 100 );
	delay_us( 100 );delay_us( 100 ); // czekanie na zakoczenie incjalizacji
	DQ=1;
} 

void BusWrite(unsigned char  data) 
{ 
char i; 

	for (i=0; i<8;++i) 
	{ 
	DQ_IN=0;	//stan"L" - incjowanie czasowej szczeliny zapisu  
	DQ=0;
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//przez 2us
	if((data&1)==1)	//wystawienie bitu na magistral 
	DQ=1;
	else
	DQ=0;
	data=data>>1;//przesunicie w prawo
	delay_us( 70 ); //czekanie na zakoczenie sczeliny czasowej zapisu  
	DQ=1;//Stan "H" na magistrali 
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//delay 2us - czas >1us 
	} 
} 

//*********************************************************************
//odczytanie 8 bitowej danej z magistrali 1-wire
//*********************************************************************
unsigned char  BusRead() 
{ 
unsigned char i,data;
	data=0;
	DQ_IN=0;
	for (i=0;i<8;i++) 
	{
	DQ=0;//pocztek szceliny czasowej
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//przez 2us
	DQ=1;//magistrala w stanie wysokim (wejcie)
	//8us na ustabilizowanie danych na magistrali
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//delay 2us; 
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//delay 2us; 
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//delay 2us; 
	asm("nop"); asm("nop"); asm("nop"); asm("nop");//delay 2us;  
	asm("nop"); asm("nop"); //asm("nop"); asm("nop");//delay 2us; 
	data=data>>1;//przesuwanie 8 bitw danych w mniennej data
	if(DQ_IN==1)
	data=(data|0x80);
	else
	data=data&0x7f;
	delay_us( 100 ); // czekanie na zakoczenie sczeliny czasowej 
	} 
	return( data ); 
}

void delay_us(unsigned char usek){
	
	PR2=usek*2;
	TMR2IF=0;
	TMR2ON=1;
	while(!TMR2IF);
	TMR2ON=0;
}
void delay_ms(unsigned int msek){
	mSec=msek;
	TMR0|=131;
	tms=1;
	while(tms);
}
